#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define MAX_LEN 15
#define LETTER_WIDTH 5
#define TEXT_LENGTH 232
#define SPACE_WIDTH 3
#define S 0  // space
#define R 1
#define G 2
#define Y 3  // RG
#define B 4
#define V 5  // RB
#define A 6  // GB
#define W 7  // RGB
#define CYCLE 302

char text[MAX_LEN] = "PM PROJECT";
char color[MAX_LEN] = {G, G, S, A, A, Y, Y, V, V, W};
int textLen = 10;

const int letters[][5] =
{
 {0xfe, 0x11, 0x11, 0x11, 0xfe}, //A
 {0xff, 0x89, 0x89, 0x89, 0x76}, //B
 {0xfe, 0x81, 0x81, 0x81, 0x42}, //C
 {0xfe, 0x81, 0x81, 0x42, 0x3c}, //D
 {0x7e, 0x89, 0x89, 0x89, 0x81}, //E
 {0xff, 0x09, 0x09, 0x09, 0x01}, //F
 {0x7e, 0x81, 0x91, 0x91, 0x76}, //G
 {0xff, 0x08, 0x08, 0x08, 0xff}, //H
 {0x00, 0x81, 0xff, 0x81, 0x00}, //I
 {0x60, 0x80, 0x81, 0x7f, 0x01}, //J
 {0xff, 0x18, 0x24, 0x42, 0x81}, //K
 {0xff, 0x80, 0x80, 0x80, 0x80}, //L
 {0xff, 0x04, 0x08, 0x04, 0xff}, //M
 {0xff, 0x06, 0x18, 0x60, 0xff}, //N
 {0x7e, 0x81, 0x81, 0x81, 0x7e}, //O
 {0xff, 0x09, 0x09, 0x09, 0x06}, //P
 {0x3e, 0x41, 0x51, 0x61, 0xbe}, //Q
 {0xff, 0x19, 0x29, 0x49, 0x86}, //R
 {0x66, 0x89, 0x89, 0x89, 0x72}, //S
 {0x01, 0x01, 0xff, 0x01, 0x01}, //T
 {0x7f, 0x80, 0x80, 0x80, 0x7f}, //U
 {0x0f, 0x70, 0x80, 0x70, 0x0f}, //V
 {0x7f, 0x80, 0x70, 0x80, 0x7f}, //W
 {0xc3, 0x24, 0x18, 0x24, 0xc3}, //X
 {0x03, 0x0c, 0xf0, 0x0c, 0x03}, //Y
 {0x81, 0xa1, 0x99, 0x85, 0x81}  //Z  
};

volatile unsigned char textR[TEXT_LENGTH];
volatile unsigned char textG[TEXT_LENGTH];
volatile unsigned char textB[TEXT_LENGTH];
volatile unsigned int arrLen = 0;
volatile unsigned int arrIx = 0;

volatile unsigned int space = 0;
volatile unsigned int space_idx = 0;

void buildTextArray();
void nextSegment();
void timer1_init();

int main() {
	sei();

	DDRA = 255;
	DDRC = 255;
	DDRD = 255;

	PORTD = 0;
	PORTC = 0;
	PORTA = 0;

	arrIx = 0;
	arrLen = 0;
	space_idx = 0;

	buildTextArray();

	timer1_init();
	while (1);
		
	return 0;
}

void buildTextArray(void) {
	int i, j;
	for (i = textLen - 1; i >= 0; i--) {
		for (j = LETTER_WIDTH - 1; j >= 0; j--) {
			// R
			if ((color[i] & 1) != 0)
				textR[arrLen] = letters[text[i] - 'A'][j];
			else
				textR[arrLen] = 0;

			// G
			if ((color[i] & 2) != 0)
				textG[arrLen] = letters[text[i] - 'A'][j];
			else
				textG[arrLen] = 0;

			// B
			if ((color[i] & 4) != 0)
				textB[arrLen] = letters[text[i] - 'A'][j];
			else
				textB[arrLen] = 0;

			arrLen++;
		
		}

		for (j = 0; j < SPACE_WIDTH; j++) {
			textR[arrLen] = 0;
			textG[arrLen] = 0;
			textB[arrLen] = 0;
			arrLen++;
		}
	}

	space = CYCLE - textLen * (LETTER_WIDTH + SPACE_WIDTH);
}
void nextSegment(void) {
	if (arrIx < arrLen) {
		PORTA = textR[arrIx];
		PORTC = textG[arrIx];
		PORTD = textB[arrIx];
		arrIx++;
	}
	else {
		if (space_idx < space)
			space_idx++;
		else {
			arrIx = 0;
			space_idx = 0;
		}
	}
}	

ISR(TIMER1_COMPA_vect) {
	nextSegment();
}

void  timer1_init() {
	OCR1A = 18;
	TCCR1A = 0;
	TCCR1B = (5 << CS10) | (1 << CS12) | (1 << WGM12);
	TIMSK1 = (1 << OCIE1A);
}
